%% This script rund the eplxanation algorithm on 20 newsgroups and assesses
% the performance on the provided test set

% Number of features to keep for each class. Since there are 20 newsgroups,
% we expect 20 * nb_indices_to_keep features to remain.
nb_indices_to_keep = 50;

% Number of instances to sample with the 1 vs all: hence sampled_nb_instances from the newsgroup and sampled_nb_instances from
% outside the newsgroup
sampled_nb_instances = 1000;

nb_test_instances_to_explain_per_newsgroup = 10;

%% Load preprocessed data
%   load, input selection, from sparse to full matrices
[X_tr, Y_tr, X_te, Y_te, feature_names, groups, groupnames] = EDC_preprocess_20newsgroups(nb_indices_to_keep);
load 20newsgroups_preprocessed;

%% For each group: create a dataset, run SVM and explain model
%cd 'C:\Documents and Settings\dmartens\My Documents\Research\Explaining Classifications\Datasets'
cd 'D:\Data folder\20newsgroups';

% Build indices with nb_test_instances_to_explain_per_newsgroup: the same
% for all newsgroups and models
% indices_to_explain = [];
% nb_groups = size(groups,1);
% s=RandStream('mt19937ar');
% RandStream.setDefaultStream(s)
% indices_to_explain = [];
% for ci = 1:20
% 	ci_i = find(Y_te == ci);
%     sel_ci_i = randpermseed(s,size(ci_i,1));
%     indices_to_explain = [indices_to_explain; ci_i(sel_ci_i(1:nb_test_instances_to_explain_per_newsgroup))];
% end
%
% Save and load such that same documen,ts each time
% save explainingdocs_indices indices_to_explain;
load explainingdocs_indices;
X_te_explain = X_te(indices_to_explain,:);
Y_te_explain = Y_te(indices_to_explain,:);


for groupi = [nb_groups:-1:1]

% CREATE THE DATASET
    ingroup = groups{groupi};
    % Eg: group 2 is computer (class 2 to 6) versus non-computer subjects
    % Take random selection of 'sampled_nb_instances' instances for each class

    indices_computer_tr = find(Y_tr >= min(ingroup) & Y_tr <= max(ingroup));
    X_tr_comp = X_tr(indices_computer_tr,:);
    Y_tr_comp = ones(size(indices_computer_tr,1),1);
    indices_noncomputer_tr = setdiff([1:size(X_tr,1)],indices_computer_tr)';
    X_tr_noncomp = X_tr(indices_noncomputer_tr,:);
    Y_tr_noncomp = - ones(size(indices_noncomputer_tr,1),1);

	% Take sample from each class
    sel_comp = randperm(size(X_tr_comp,1));
    sel_noncomp = randperm(size(X_tr_noncomp,1));
    
    % In some cases less than 1000 to sample
    sampled_nb_instances2 = min(sampled_nb_instances,size(sel_comp,2));
    
    X_tr_sample = [X_tr_comp(sel_comp(1:sampled_nb_instances2),:); X_tr_noncomp(sel_noncomp(1:sampled_nb_instances),:)];
    Y_tr_sample = [Y_tr_comp(sel_comp(1:sampled_nb_instances2),:); Y_tr_noncomp(sel_noncomp(1:sampled_nb_instances),:)];

    % From class 1 to 20 to class -1 and 1
    indices_computer_te = find(Y_te_explain >= min(ingroup) & Y_te_explain <= max(ingroup));
    indices_noncomputer_te = setdiff([1:size(Y_te_explain,1)],indices_computer_te)';
    Y_te_explain2 = zeros(size(Y_te_explain,1),1);
    Y_te_explain2(indices_computer_te) = 1;
    Y_te_explain2(indices_noncomputer_te) = -1;
    
    
% RUN LS-SVM AN EXPLAIN
    output_f_name_begin = groupnames{groupi};
    [perf, perf1, perf0, all_explanations] = EDC(X_tr_sample, Y_tr_sample, X_te_explain, Y_te_explain2, Y_te_explain, feature_names, output_f_name_begin);
    feval('save',strcat('explained_',groupnames{groupi}));
    
end