function [t1, t2, iteration_first_expl, word_combos_explained] = EDC_libSVM_explain_doc(docu, doc_nb, class_orig, feature_names, output_file, bb_eval, bb_eval_args);

max_nb_of_words = 30;
max_nb_of_iterations = 30;


%% Explains trained classification model for the given document
t1 = NaN;
t2 = NaN;
iteration_first_expl = NaN;
warning off
fid = fopen(output_file ,'a');
tic;
non_zero_features = find(docu ~= 0);
fprintf(1,'\n  ',int2str(doc_nb));
fprintf(fid,'\n  ',int2str(doc_nb));
for wi=1:size(non_zero_features,2)
    fprintf(1,' %s ',cell2mat(feature_names(non_zero_features(wi))));
    fprintf(fid,' %s ',cell2mat(feature_names(non_zero_features(wi))));
end
[c acc p] = feval(bb_eval,1,sparse(docu),bb_eval_args,'-b 1');
% See Q reveresed on http://www.csie.ntu.edu.tw/~cjlin/libsvm/faq.html#f430
p = p*bb_eval_args.Label(1);

fprintf(1,'\nExplaining document %d (class %d) with %d features and class %d (score %g)...\n', doc_nb, class_orig, size(non_zero_features,2),c,p);
fprintf(fid,'\nExplaining document %d (class %d) with %d features and class %d (score %g)...\n', doc_nb, class_orig, size(non_zero_features,2),c,p);

doc_new_temp = docu;
word_combos_explained = {};
nb_combos_explained = 0 ;
combinations_to_expand_on = [];
probs_combinations_to_expand_on = [];

iteration = 0;
while((iteration < max_nb_of_iterations) && (nb_combos_explained < 10))
    iteration = iteration + 1;
	fprintf(1,'\nIteration %d \n', iteration);
    % Create combination of nb_of_words words
    % Can limit ourselves to the non-zero features
    %if (nb_of_words  <= size(non_zero_features,2))
    % Create word combinations

    if (c == 1)
        % Take largest decrease in p
        [max_delta,max_delta_i] = max(p - probs_combinations_to_expand_on);
    elseif (c == -1)
        % Take largest increase in p
        [max_delta,max_delta_i] = max(probs_combinations_to_expand_on - p);
    else error('We assume class being -1 or 1!');
    end

    % Expand on the 'max_delta_i'th combination:
    % 1. Create new word combinations
    if (size(max_delta_i,1) == 0)
        % No combinations exist yet: create combinations of 1 word
        new_combos_max_delta = zeros(size(non_zero_features,2),max_nb_of_words);
        for i=1:size(non_zero_features,2)
            new_combos_max_delta(i,1) = non_zero_features(i);
        end
    else
        nb_vars_in_combo = size(find(combinations_to_expand_on(max_delta_i,:)~= 0),2);
        vars_to_add = setdiff(non_zero_features,combinations_to_expand_on(max_delta_i,:));
        new_combos_max_delta = zeros(size(vars_to_add,2),max_nb_of_words);
        for add_var_i = 1:size(vars_to_add,2)
            new_combos_max_delta(add_var_i,:) = combinations_to_expand_on(max_delta_i,:);
            new_combos_max_delta(add_var_i,nb_vars_in_combo+1) = vars_to_add(add_var_i);
            new_combos_max_delta(add_var_i,:) = sort(new_combos_max_delta(add_var_i,:),'descend');
        end
    end
    % Remove the explained combinations
    new_combos_max_delta2 = remove_combinations(new_combos_max_delta,word_combos_explained);

    % 2. Classify new word combinations
    word_combos = new_combos_max_delta2;
    probs = zeros(size(word_combos,1),1);
    for i=1:size(word_combos,1)
        doc_new = doc_new_temp;
        word_combo = word_combos(i,:);
        for word=word_combo
            if (word ~= 0)
                doc_new(word) = 0;
            end
        end
        [c_new, acc_new, p_new] = feval(bb_eval,1,sparse(doc_new),bb_eval_args,'-b 1');
        p_new = p_new*bb_eval_args.Label(1);
        probs(i) = p_new;
        if (c_new ~= c)
            if(nb_combos_explained  == 0)
                t1 = toc;
                iteration_first_expl = iteration;
            end
            nb_combos_explained = nb_combos_explained + 1;
            word_combos_explained(nb_combos_explained) = mat2cell(word_combo);
            fprintf(1,'doc %d (from score %g to %g): If words (',doc_nb,p,p_new);
            fprintf(fid,'Iteration %d (from score %g to %g): If words (',iteration,p,p_new);
            nb_of_words = size(find(word_combo > 0),2);
            for k=1:nb_of_words
                fprintf(1,'%s ',cell2mat(feature_names(word_combo(k))));
                fprintf(fid,'%s ',cell2mat(feature_names(word_combo(k))));
            end
            t2=toc;
            fprintf(1,') are removed then class changes from %d to %d \n',c,c_new);
            fprintf(fid,') are removed then class changes from %d to %d (%d sec)\n',c,c_new,round(t2));
        end
    end

    % 3. If an explanation is found, remove this combination from
    % combinations to expand on. Also remove all combinations that can
    % not be expanded upon as the max nb of words is reached.
    [word_combos,probs] = remove_combinations2(word_combos,word_combos_explained,probs,max_nb_of_words);


    % 4. Add new word combinations to set of existing combinations
    % that were tried and can be expanded on, and remove the combination that was just expanded on.
    [s1,s2] = size(combinations_to_expand_on);
    % Remove combination that was just expanded on.
    if(s1>0)
        % If first expansion: no combination existed yet.
        temp = zeros(s1-1,s2);
        temp(1:max_delta_i-1,:) = combinations_to_expand_on(1:max_delta_i-1,:);
        temp(max_delta_i:s1-1,:) = combinations_to_expand_on(max_delta_i+1:s1,:);

        temp_probs = zeros(s1-1,1);
        temp_probs(1:max_delta_i-1,:) = probs_combinations_to_expand_on(1:max_delta_i-1,:);
        temp_probs(max_delta_i:s1-1,:) = probs_combinations_to_expand_on(max_delta_i+1:s1,:);
    else
        temp = []; temp_probs = [];
    end
    combinations_to_expand_on = [temp;word_combos];
    probs_combinations_to_expand_on = [temp_probs;probs];

end
if (nb_combos_explained == 0)
    if(size(probs_combinations_to_expand_on,1)~=0)
        fprintf(fid,'No explanations: score dropped to %g\n',probs_combinations_to_expand_on(max_delta_i));
    else
        fprintf(fid,'No explanations: no words that increase the score: ');
        for iii=1:size(non_zero_features,2)
            fprintf(fid,' %s  (w=%g) ',cell2mat(feature_names(non_zero_features(iii))), bb_eval_args.w(non_zero_features(iii)));
        end
    end
end
fclose(fid);
save forplottinghelp
